/**
 * @file
 *
 * @copyright Axiros GmbH 2019-2021, All rights reserved
 *
 * @brief Functionality for managing notifications definitions
 */

#ifndef CORE_NOTIFICATIONS_H_
#define CORE_NOTIFICATIONS_H_

#include "compiler.h"

/* Forward declarations */
struct notifications;
struct notification_entry;

typedef void (*notification_cb) (struct notification_entry *entry, void *data);

enum notification_type {
	EVENT_UNKNOWN = 0,
	EVENT_ONBOARDREQUEST = 1,
	EVENT_BOOTSTRAP = 2,
	EVENT_BOOT = 3,
	EVENT_VALUE_CHANGE = 4,
	EVENT_OBJECT_CREATION = 5,
	EVENT_OBJECT_DELETION = 6,
	EVENT_EVENT = 7,
};

/**
 * @brief Initialises a new empty notifications list
 *
 * @return a freshly allocated notifications list
 */
struct notifications *notifications_init (void) MALLOC RETURNS_NONNULL;


/**
 * @brief Add a notification subscriber to the notification list
 *
 * @param[in] list	the notification list to add the subscriber to
 * @param[in] cb	the callback to execute for notification events
 * @param[in] data	additional data to pass to subscriber when callback is executed
 */
void notifications_add_subscriber (struct notifications *list, notification_cb cb, void *data) ARGINDEX_NONNULL(1) ARGINDEX_NONNULL(2);


/**
 * @brief Add a new notification entry to the notification list
 *
 * @param[in] list	the notification list to add the entry to
 *
 * @return a freshly allocated notifications entry
 */
struct notification_entry *notifications_add_entry (struct notifications *list) ARGS_NONNULL RETURNS_NONNULL MALLOC;


/**
 * @brief Create a new notification entry and add it to the notification list
 *
 * @param[in] list	the notification list to add the entry to
 * @param[in] type	the type of notification to add
 * @param[in] data	data specific notification data
 *
 * @return a freshly allocated notifications entry
 */
struct notification_entry *notifications_create_entry (struct notifications *list, enum notification_type type, void *data) ARGINDEX_NONNULL(1) RETURNS_NONNULL MALLOC;


/**
 * @brief Get the notification type from the provided entry
 *
 * @param[in] entry	the notification entry to get the type from
 *
 * @return the type of this particular notification entry
 */
enum notification_type notification_get_type (struct notification_entry *entry) ARGS_NONNULL;


/**
 * @brief Get the associated data from the from provided entry
 *
 * @param[in] entry	the notification entry to get the data from
 *
 * @return the data of this particular notification entry, can be NULL if not associated data is available
 */
void *notification_get_data (struct notification_entry *entry) ARGS_NONNULL;


/**
 * @brief Retrieve the first entry from the notifications list
 *
 * @param[in] list	the notification list to retrieve the first entry from
 *
 * @return the first notification entry from the list or NULL if the list is empty
 */
struct notification_entry *notifications_get_first_entry (struct notifications *list) ARGS_NONNULL;


/**
 * @brief Process all pending notifications by executing all subscriber callbacks on each event, then removing it
 *
 * @param[in] list	the notification list to process
 */
void notifications_process (struct notifications *list) ARGS_NONNULL;


/**
 * @brief Remove the specified notification entry from the notification list
 *
 * @param[in] list	the notification list to remove an entry from
 * @param[in] entry	the entry to remove from the list
 */
void notifications_remove_entry (struct notifications *list, struct notification_entry *entry);


/**
 * @brief Free the notification list and all of its entries
 *
 * @param[in] list	the notification list to free
 */
void notifications_free (struct notifications *list) ARGS_NONNULL;

#endif /* CORE_NOTIFICATIONS_H_ */
